import ddf.minim.*;
import ddf.minim.signals.*;
import controlP5.*;

ControlP5 controlP5;

int numCarsToStart = 20;
int trackWidth = 20;
int middleX = 0;
int middleY = 0;
float twoPI = (float)(Math.PI * 2);
float targetSpeed = 0.02;
float dangerRadians = twoPI / 60;

ArrayList cars = new ArrayList();

Minim minim;
AudioOutput output;
SineWave lapBleep;

void setup()
{
  println("setting up...");

  size(400, 400); 
  smooth();

  middleX = width - trackWidth;
  middleY = height - trackWidth;

  frameRate(8);

  minim = new Minim(this);
  output = minim.getLineOut(Minim.STEREO);
  lapBleep = new SineWave(1000, 0.0, 44100);
  output.addSignal(lapBleep);

  for (int i = 0; i < numCarsToStart; i++)
  {
    //figure out starting points
    float angle = i * (twoPI/numCarsToStart);

    if (i % 2 == 0)
      cars.add(new SynthCar(i, angle, targetSpeed, output));
    else
      cars.add(new SampleCar(i, angle, targetSpeed));
  }

  controlP5 = new ControlP5(this);

  controlP5.addButton("plus",10,width/2 - 60, height/2 - 25,50,50).setId(1);
  controlP5.addButton("minus",11,width/2 + 10, height/2 - 25,50,50).setId(2);
}

void draw()
{  
  background(0);
  stroke(255);
  noFill();
  ellipse(width/2, height/2,width,height);
  ellipse(width/2, height/2, width - 40, height - 40);

  lapBleep.setAmp(0.0);

  for (int i = 0; i < cars.size(); i++)
  {
    Car car = (Car)cars.get(i);
    Car carInFront;

    if (i == cars.size() - 1)
      carInFront = (Car)cars.get(0);
    else
      carInFront = (Car)cars.get(i+1);

    car.drive(carInFront);
    car.draw(); 

    if (car.justWrapped)
    {
      lapBleep.setAmp(0.25);
      stroke(255);
      ellipse(width/2, height - trackWidth, 10, 10);
    }
  }

}

//not doing anything useful here...
void controlEvent(ControlEvent theControlEvent) 
{
  int id = theControlEvent.controller().id();
  
  println("got controlEvent: " + id);
}


void plus(int theValue) 
{
    Car lastCar = (Car)cars.get(cars.size() - 1);

    float lastAngle = lastCar.angle;
    float newAngle = lastAngle + 0.02;

    if (cars.size() % 2 == 0)
      cars.add(new SynthCar(cars.size(), newAngle, targetSpeed, output));
    else
      cars.add(new SampleCar(cars.size(), newAngle, targetSpeed));
      
    println("adding a new car!");
}

void minus(int theValue) 
{
    cars.remove(cars.size() - 1);
    println("removing a car!");
}
////////////////////

void stop()
{
  output.close();
  minim.stop();
  super.stop();
}


